import os
import sys
import re
from viyu.commands.create_server import create_server
from viyu.commands.add_app import add_app
from viyu.commands.package_utils import create_package, build_package

def is_valid_name(name):
    """Check if the provided name is valid (alphanumeric with underscores)."""
    return bool(re.match(r"^[a-zA-Z0-9_]+$", name))

def display_welcome_message():
    """Display the welcome message and available commands."""
    print("\n✨ Welcome to Viyu CLI! ✨")
    print("We’re thrilled to have you here! Viyu is your personal framework to simplify server, app, and package creation.\n")
    print("Here’s how you can use Viyu CLI:")
    print("1️⃣ `viyu new <project_name>`   : Create a new server project.")
    print("2️⃣ `viyu add <app_name>`       : Add a new app to an existing project.")
    print("3️⃣ `viyu package <package_name>` : Create a new Python package.")
    print("4️⃣ `viyu build <package_name>`   : Build an existing Python package.")
    print("\n📖 Documentation: https://viyu-docs.example.com")
    print("🚀 Let’s get started! 🚀\n")

def display_help():
    """Display help message with available commands."""
    print("\n✨ Viyu CLI Help ✨")
    print("Available commands:")
    print("1️⃣ `viyu new <project_name>`   : Create a new server project.")
    print("2️⃣ `viyu add <app_name>`       : Add a new app to an existing project.")
    print("3️⃣ `viyu package <package_name>` : Create a new Python package.")
    print("4️⃣ `viyu build <package_name>`   : Build an existing Python package.")
    print("\n📖 Documentation: https://viyu-docs.example.com")
    print("🚀 Let’s get started! 🚀\n")

def create_project(project_name):
    """Handle the creation of a new project."""
    if not is_valid_name(project_name):
        print("❌ Invalid project name! Use only letters, numbers, and underscores.")
        return
    try:
        create_server(project_name)
        print(f"✅ Project `{project_name}` created successfully!")
    except Exception as e:
        print(f"❌ Failed to create project `{project_name}`. Error: {str(e)}")

def add_application(app_name):
    """Handle the addition of a new app to the current project."""
    if not is_valid_name(app_name):
        print("❌ Invalid app name! Use only letters, numbers, and underscores.")
        return
    try:
        current_directory = os.getcwd()
        app_folder = os.path.join(current_directory, "apps", app_name)
        os.makedirs(app_folder, exist_ok=True)
        add_app(app_name, app_folder)
        print(f"✅ App `{app_name}` created successfully!")
    except Exception as e:
        print(f"❌ Failed to create app `{app_name}`. Error: {str(e)}")

def create_python_package(package_name):
    """Create a new Python package."""
    if not is_valid_name(package_name):
        print("❌ Invalid package name! Use only letters, numbers, and underscores.")
        return
    try:
        create_package(package_name)
        print(f"✅ Python package `{package_name}` created successfully!")
    except Exception as e:
        print(f"❌ Failed to create package `{package_name}`. Error: {str(e)}")

def build_python_package(package_name):
    """Build an existing Python package."""
    if not is_valid_name(package_name):
        print("❌ Invalid package name! Use only letters, numbers, and underscores.")
        return
    try:
        build_package(package_name)
        print(f"✅ Python package `{package_name}` built successfully!")
    except Exception as e:
        print(f"❌ Failed to build package `{package_name}`. Error: {str(e)}")

def main():
    try:
        if len(sys.argv) == 2 and sys.argv[1] in ["--help", "help"]:
            display_help()
            return
        
        if len(sys.argv) == 1:
            display_welcome_message()
            return
        
        command = sys.argv[1]
        commands = {
            "new": create_project,
            "add": add_application,
            "package": create_python_package,
            "build": build_python_package
        }

        if command in commands and len(sys.argv) == 3:
            commands[command](sys.argv[2])
        else:
            print(f"❌ Unknown or invalid command `{command}`.")
            display_welcome_message()
    except KeyboardInterrupt:
        print("\n❌ Operation cancelled by user.")
    except Exception as e:
        print(f"❌ An unexpected error occurred: {str(e)}")
    finally:
        print("\n✨ Thank you for using Viyu! ✨\n")

if __name__ == "__main__":
    main()

