import os
import shutil
import subprocess

def create_package(package_name):
    """
    Create a new Python package with the given name.
    """
    try:
        if os.path.exists(package_name):
            print(f"❌ Package `{package_name}` already exists.")
            return
        
        os.makedirs(package_name)
        os.makedirs(os.path.join(package_name, package_name))
        
        # Create basic package files
        with open(os.path.join(package_name, "setup.py"), "w") as f:
            f.write(f"""
from setuptools import setup, find_packages

setup(
    name='{package_name}',
    version='0.1.0',
    packages=find_packages(),
    install_requires=[],
)
""".strip())

        with open(os.path.join(package_name, package_name, "__init__.py"), "w") as f:
            f.write("""
# {package_name} package initialization file
""".strip())
        
        with open(os.path.join(package_name, "README.md"), "w") as f:
            f.write(f"# {package_name}\n\nThis is a Python package.")
        
        with open(os.path.join(package_name, "requirements.txt"), "w") as f:
            f.write("""
# Add dependencies here
""".strip())
        
        print(f"✅ Package `{package_name}` created successfully!")
    except Exception as e:
        print(f"❌ Failed to create package `{package_name}`. Error: {str(e)}")

def build_package(package_name):
    """
    Build the Python package.
    """
    try:
        if not os.path.exists(package_name):
            print(f"❌ Package `{package_name}` does not exist.")
            return
        
        os.chdir(package_name)
        
        # Ensure build tools are installed
        subprocess.run(["pip", "install", "setuptools", "wheel"], check=True)
        
        # Run the build command
        subprocess.run(["python", "setup.py", "sdist", "bdist_wheel"], check=True)
        
        print(f"✅ Package `{package_name}` built successfully!")
    except Exception as e:
        print(f"❌ Failed to build package `{package_name}`. Error: {str(e)}")
    finally:
        os.chdir("..")
